import os
import sys
import time
from distutils.version import LooseVersion

import pytest

if sys.platform == "linux":
    print("Linux system detected")
    import pynvml

    NVML_PCIE_UTIL_TX_BYTES = pynvml.NVML_PCIE_UTIL_TX_BYTES
    NVML_PCIE_UTIL_RX_BYTES = pynvml.NVML_PCIE_UTIL_RX_BYTES
    NVML_PCIE_UTIL_COUNT = pynvml.NVML_PCIE_UTIL_COUNT

    # Test pynvml.nvmlDeviceGetNvLinkCapability

    @pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
    @pytest.mark.parametrize(
        "cap_type",
        [
            pynvml.NVML_NVLINK_CAP_P2P_SUPPORTED,  # P2P over NVLink is supported
            pynvml.NVML_NVLINK_CAP_SYSMEM_ACCESS,  # Access to system memory is supported
            pynvml.NVML_NVLINK_CAP_P2P_ATOMICS,  # P2P atomics are supported
            pynvml.NVML_NVLINK_CAP_SYSMEM_ATOMICS,  # System memory atomics are supported
            pynvml.NVML_NVLINK_CAP_SLI_BRIDGE,  # SLI is supported over this link
            pynvml.NVML_NVLINK_CAP_VALID,
        ],
    )  # Link is supported on this device
    def test_nvml_nvlink_capability(ngpus, handles, cap_type):
        for i in range(ngpus):
            for j in range(pynvml.NVML_NVLINK_MAX_LINKS):
                cap = pynvml.nvmlDeviceGetNvLinkCapability(handles[i], j, cap_type)
                assert cap >= 0

    # Test pynvml.nvmlDeviceResetNvLinkErrorCounters
    # Test pynvml.nvmlDeviceGetNvLinkErrorCounter
    @pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
    @pytest.mark.parametrize(
        "error_type",
        [
            pynvml.NVML_NVLINK_ERROR_DL_REPLAY,
            pynvml.NVML_NVLINK_ERROR_DL_RECOVERY,
            pynvml.NVML_NVLINK_ERROR_DL_CRC_FLIT,
            pynvml.NVML_NVLINK_ERROR_DL_CRC_DATA,
        ],
    )
    def test_nvml_nvlink_error_counters(ngpus, handles, error_type):
        for i in range(ngpus):
            for j in range(pynvml.NVML_NVLINK_MAX_LINKS):
                assert (
                    pynvml.nvmlDeviceResetNvLinkErrorCounters(handles[i], j)
                    == pynvml.NVML_SUCCESS
                )
                error_count = pynvml.nvmlDeviceGetNvLinkErrorCounter(
                    handles[i], j, error_type
                )
                assert error_count >= 0


# Fixture to initialize and finalize nvml
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
@pytest.fixture(scope="module")
def nvml(request):
    """ """
    pynvml.nvmlInit()

    def nvml_close():
        """ """
        pynvml.nvmlShutdown()

    request.addfinalizer(nvml_close)


# Get GPU count
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
@pytest.fixture
def ngpus(nvml):
    """ """
    result = pynvml.nvmlDeviceGetCount()
    assert result > 0
    print("[" + str(result) + " GPUs]", end=" ")
    return result


# Get handles using pynvml.nvmlDeviceGetHandleByIndex
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
@pytest.fixture
def handles(ngpus):
    """ """
    handles = [pynvml.nvmlDeviceGetHandleByIndex(i) for i in range(ngpus)]
    assert len(handles) == ngpus
    return handles


@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
@pytest.fixture
def serials(ngpus, handles):
    """ """
    serials = [pynvml.nvmlDeviceGetSerial(handles[i]) for i in range(ngpus)]
    assert len(serials) == ngpus
    return serials


@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
@pytest.fixture
def uuids(ngpus, handles):
    """ """
    uuids = [pynvml.nvmlDeviceGetUUID(handles[i]) for i in range(ngpus)]
    assert len(uuids) == ngpus
    return uuids


@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
@pytest.fixture
def pci_info(ngpus, handles):
    """ """
    pci_info = [pynvml.nvmlDeviceGetPciInfo(handles[i]) for i in range(ngpus)]
    assert len(pci_info) == ngpus
    return pci_info


## ---------------------------- ##
## Start Fucntion-pointer Tests ##
## ---------------------------- ##

# Test pynvml.nvmlSystemGetNVMLVersion
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlSystemGetNVMLVersion(nvml):
    vsn = 0.0
    vsn = pynvml.nvmlSystemGetNVMLVersion().decode()
    print("[NVML Version: " + vsn + "]", end=" ")
    assert vsn > LooseVersion("0.0")


# Test pynvml.nvmlSystemGetProcessName
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlSystemGetProcessName(nvml):
    procname = None
    procname = pynvml.nvmlSystemGetProcessName(os.getpid())
    print("[Process: " + str(procname.decode()) + "]", end=" ")
    assert procname != None


# Test pynvml.nvmlSystemGetDriverVersion
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlSystemGetDriverVersion(nvml):
    vsn = 0.0
    vsn = pynvml.nvmlSystemGetDriverVersion().decode()
    print("[Driver Version: " + vsn + "]", end=" ")
    assert vsn > LooseVersion("0.0")  # Developing with 396.44


## Unit "Get" Functions (Skipping for now) ##

## Device "Get" Functions (Note: Some functions are fixtures, above) ##

# Test pynvml.nvmlDeviceGetHandleBySerial
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlDeviceGetHandleBySerial(ngpus, serials):
    handles = [pynvml.nvmlDeviceGetHandleBySerial(serials[i]) for i in range(ngpus)]
    assert len(handles) == ngpus


# Test pynvml.nvmlDeviceGetHandleByUUID
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlDeviceGetHandleByUUID(ngpus, uuids):
    handles = [pynvml.nvmlDeviceGetHandleByUUID(uuids[i]) for i in range(ngpus)]
    assert len(handles) == ngpus


# Test pynvml.nvmlDeviceGetHandleByPciBusId
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlDeviceGetHandleByPciBusId(ngpus, pci_info):
    handles = [
        pynvml.nvmlDeviceGetHandleByPciBusId(pci_info[i].busId) for i in range(ngpus)
    ]
    assert len(handles) == ngpus


# [Skipping] pynvml.nvmlDeviceGetName
# [Skipping] pynvml.nvmlDeviceGetBoardId
# [Skipping] pynvml.nvmlDeviceGetMultiGpuBoard
# [Skipping] pynvml.nvmlDeviceGetBrand
# [Skipping] pynvml.nvmlDeviceGetCpuAffinity
# [Skipping] pynvml.nvmlDeviceSetCpuAffinity
# [Skipping] pynvml.nvmlDeviceClearCpuAffinity
# [Skipping] pynvml.nvmlDeviceGetMinorNumber
# [Skipping] pynvml.nvmlDeviceGetUUID
# [Skipping] pynvml.nvmlDeviceGetInforomVersion
# [Skipping] pynvml.nvmlDeviceGetInforomImageVersion
# [Skipping] pynvml.nvmlDeviceGetInforomConfigurationChecksum
# [Skipping] pynvml.nvmlDeviceValidateInforom
# [Skipping] pynvml.nvmlDeviceGetDisplayMode
# [Skipping] pynvml.nvmlDeviceGetPersistenceMode
# [Skipping] pynvml.nvmlDeviceGetClockInfo
# [Skipping] pynvml.nvmlDeviceGetMaxClockInfo
# [Skipping] pynvml.nvmlDeviceGetApplicationsCloc
# [Skipping] pynvml.nvmlDeviceGetDefaultApplicationsClock
# [Skipping] pynvml.nvmlDeviceGetSupportedMemoryClocks
# [Skipping] pynvml.nvmlDeviceGetSupportedGraphicsClocks
# [Skipping] pynvml.nvmlDeviceGetFanSpeed
# [Skipping] pynvml.nvmlDeviceGetTemperature
# [Skipping] pynvml.nvmlDeviceGetTemperatureThreshold
# [Skipping] pynvml.nvmlDeviceGetPowerState
# [Skipping] pynvml.nvmlDeviceGetPerformanceState
# [Skipping] pynvml.nvmlDeviceGetPowerManagementMode
# [Skipping] pynvml.nvmlDeviceGetPowerManagementLimit
# [Skipping] pynvml.nvmlDeviceGetPowerManagementLimitConstraints
# [Skipping] pynvml.nvmlDeviceGetPowerManagementDefaultLimit
# [Skipping] pynvml.nvmlDeviceGetEnforcedPowerLimit

# Test pynvml.nvmlDeviceGetPowerUsage
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlDeviceGetPowerUsage(ngpus, handles):
    for i in range(ngpus):
        power_mWatts = pynvml.nvmlDeviceGetPowerUsage(handles[i])
        assert power_mWatts >= 0.0


# Test pynvml.nvmlDeviceGetTotalEnergyConsumption
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlDeviceGetTotalEnergyConsumption(ngpus, handles):
    for i in range(ngpus):
        energy_mJoules1 = pynvml.nvmlDeviceGetTotalEnergyConsumption(handles[i])
        for j in range(10):  # idle for 150 ms
            time.sleep(0.015)  # and check for increase every 15 ms
            energy_mJoules2 = pynvml.nvmlDeviceGetTotalEnergyConsumption(handles[i])
            assert energy_mJoules2 >= energy_mJoules1
            if energy_mJoules2 > energy_mJoules1:
                break
        else:
            assert False, "energy did not increase across 150 ms interval"


# [Skipping] pynvml.nvmlDeviceGetGpuOperationMode
# [Skipping] pynvml.nvmlDeviceGetCurrentGpuOperationMode
# [Skipping] pynvml.nvmlDeviceGetPendingGpuOperationMode

# Test pynvml.nvmlDeviceGetMemoryInfo
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlDeviceGetMemoryInfo(ngpus, handles):
    for i in range(ngpus):
        meminfo = pynvml.nvmlDeviceGetMemoryInfo(handles[i])
        assert (meminfo.used <= meminfo.total) and (meminfo.free <= meminfo.total)


# [Skipping] pynvml.nvmlDeviceGetBAR1MemoryInfo
# [Skipping] pynvml.nvmlDeviceGetComputeMode
# [Skipping] pynvml.nvmlDeviceGetEccMode
# [Skipping] pynvml.nvmlDeviceGetCurrentEccMode (Python API Addition)
# [Skipping] pynvml.nvmlDeviceGetPendingEccMode (Python API Addition)
# [Skipping] pynvml.nvmlDeviceGetTotalEccErrors
# [Skipping] pynvml.nvmlDeviceGetDetailedEccErrors
# [Skipping] pynvml.nvmlDeviceGetMemoryErrorCounter

# Test pynvml.nvmlDeviceGetUtilizationRates
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlDeviceGetUtilizationRates(ngpus, handles):
    for i in range(ngpus):
        urate = pynvml.nvmlDeviceGetUtilizationRates(handles[i])
        assert urate.gpu >= 0
        assert urate.memory >= 0


# [Skipping] pynvml.nvmlDeviceGetEncoderUtilization
# [Skipping] pynvml.nvmlDeviceGetDecoderUtilization
# [Skipping] pynvml.nvmlDeviceGetPcieReplayCounter
# [Skipping] pynvml.nvmlDeviceGetDriverModel
# [Skipping] pynvml.nvmlDeviceGetCurrentDriverModel
# [Skipping] pynvml.nvmlDeviceGetPendingDriverModel
# [Skipping] pynvml.nvmlDeviceGetVbiosVersion
# [Skipping] pynvml.nvmlDeviceGetComputeRunningProcesses
# [Skipping] pynvml.nvmlDeviceGetGraphicsRunningProcesses
# [Skipping] pynvml.nvmlDeviceGetAutoBoostedClocksEnabled
# [Skipping] nvmlUnitSetLedState
# [Skipping] pynvml.nvmlDeviceSetPersistenceMode
# [Skipping] pynvml.nvmlDeviceSetComputeMode
# [Skipping] pynvml.nvmlDeviceSetEccMode
# [Skipping] pynvml.nvmlDeviceClearEccErrorCounts
# [Skipping] pynvml.nvmlDeviceSetDriverModel
# [Skipping] pynvml.nvmlDeviceSetAutoBoostedClocksEnabled
# [Skipping] pynvml.nvmlDeviceSetDefaultAutoBoostedClocksEnabled
# [Skipping] pynvml.nvmlDeviceSetApplicationsClocks
# [Skipping] pynvml.nvmlDeviceResetApplicationsClocks
# [Skipping] pynvml.nvmlDeviceSetPowerManagementLimit
# [Skipping] pynvml.nvmlDeviceSetGpuOperationMode
# [Skipping] nvmlEventSetCreate
# [Skipping] pynvml.nvmlDeviceRegisterEvents
# [Skipping] pynvml.nvmlDeviceGetSupportedEventTypes
# [Skipping] nvmlEventSetWait
# [Skipping] nvmlEventSetFree
# [Skipping] pynvml.nvmlDeviceOnSameBoard
# [Skipping] pynvml.nvmlDeviceGetCurrPcieLinkGeneration
# [Skipping] pynvml.nvmlDeviceGetMaxPcieLinkGeneration
# [Skipping] pynvml.nvmlDeviceGetCurrPcieLinkWidth
# [Skipping] pynvml.nvmlDeviceGetMaxPcieLinkWidth
# [Skipping] pynvml.nvmlDeviceGetSupportedClocksThrottleReasons
# [Skipping] pynvml.nvmlDeviceGetCurrentClocksThrottleReasons
# [Skipping] pynvml.nvmlDeviceGetIndex
# [Skipping] pynvml.nvmlDeviceGetAccountingMode
# [Skipping] pynvml.nvmlDeviceSetAccountingMode
# [Skipping] pynvml.nvmlDeviceClearAccountingPids
# [Skipping] pynvml.nvmlDeviceGetAccountingStats
# [Skipping] pynvml.nvmlDeviceGetAccountingPids
# [Skipping] pynvml.nvmlDeviceGetAccountingBufferSize
# [Skipping] pynvml.nvmlDeviceGetRetiredPages
# [Skipping] pynvml.nvmlDeviceGetRetiredPagesPendingStatus
# [Skipping] pynvml.nvmlDeviceGetAPIRestriction
# [Skipping] pynvml.nvmlDeviceSetAPIRestriction
# [Skipping] pynvml.nvmlDeviceGetBridgeChipInfo
# [Skipping] pynvml.nvmlDeviceGetSamples
# [Skipping] pynvml.nvmlDeviceGetViolationStatus

# Test pynvml.nvmlDeviceGetPcieThroughput
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvmlDeviceGetPcieThroughput(ngpus, handles):
    for i in range(ngpus):
        tx_bytes_tp = pynvml.nvmlDeviceGetPcieThroughput(
            handles[i], NVML_PCIE_UTIL_TX_BYTES
        )
        assert tx_bytes_tp >= 0
        rx_bytes_tp = pynvml.nvmlDeviceGetPcieThroughput(
            handles[i], NVML_PCIE_UTIL_RX_BYTES
        )
        assert rx_bytes_tp >= 0
        count_tp = pynvml.nvmlDeviceGetPcieThroughput(handles[i], NVML_PCIE_UTIL_COUNT)
        assert count_tp >= 0


# [Skipping] pynvml.nvmlSystemGetTopologyGpuSet
# [Skipping] pynvml.nvmlDeviceGetTopologyNearestGpus
# [Skipping] pynvml.nvmlDeviceGetTopologyCommonAncestor

# Test pynvml.nvmlDeviceGetNvLinkVersion
# Test pynvml.nvmlDeviceGetNvLinkState
# Test pynvml.nvmlDeviceGetNvLinkRemotePciInfo
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
def test_nvml_nvlink_properties(ngpus, handles):
    for i in range(ngpus):
        for j in range(pynvml.NVML_NVLINK_MAX_LINKS):
            version = pynvml.nvmlDeviceGetNvLinkVersion(handles[i], j)
            assert version >= 1
            state = pynvml.nvmlDeviceGetNvLinkState(handles[i], j)
            assert state >= 0
            pci_info = pynvml.nvmlDeviceGetNvLinkRemotePciInfo(handles[i], j)
            assert isinstance(pci_info, pynvml.c_nvmlPciInfo_t)


# Test pynvml.nvmlDeviceResetNvLinkUtilizationCounter
# Test pynvml.nvmlDeviceSetNvLinkUtilizationControl
# Test pynvml.nvmlDeviceGetNvLinkUtilizationCounter
# Test pynvml.nvmlDeviceGetNvLinkUtilizationControl
# Test pynvml.nvmlDeviceFreezeNvLinkUtilizationCounter
@pytest.mark.skipif(sys.platform != "linux", reason="Test only on linux")
@pytest.mark.parametrize("counter", [0, 1])
@pytest.mark.parametrize("control", [0, 1, 2])
def test_nvml_nvlink_counters(ngpus, handles, counter, control):
    reset = 0
    for i in range(ngpus):
        for j in range(pynvml.NVML_NVLINK_MAX_LINKS):
            assert (
                pynvml.nvmlDeviceResetNvLinkUtilizationCounter(handles[i], j, counter)
                == pynvml.NVML_SUCCESS
            )
            pynvml.nvmlDeviceSetNvLinkUtilizationControl(
                handles[i], j, counter, control, reset
            )
            countdict = pynvml.nvmlDeviceGetNvLinkUtilizationCounter(
                handles[i], j, counter
            )
            ctl = pynvml.nvmlDeviceGetNvLinkUtilizationControl(handles[i], j, counter)
            assert countdict["rx"] >= 0
            assert countdict["tx"] >= 0
            assert ctl == control
            assert (
                pynvml.nvmlDeviceFreezeNvLinkUtilizationCounter(
                    handles[i], j, counter, 1
                )
                == pynvml.NVML_SUCCESS
            )
            assert (
                pynvml.nvmlDeviceFreezeNvLinkUtilizationCounter(
                    handles[i], j, counter, 0
                )
                == pynvml.NVML_SUCCESS
            )



# https://github.com/pulls?q=is%3Aopen+is%3Apr+user%3Aaivclab+archived%3Afalse+
# https://github.com/pulls?user=aivclab
from datetime import datetime, timedelta
from enum import Enum
from typing import Generator, Iterable, List, Union

from github import Github  # pip install PyGithub
from github.GithubObject import NotSet
from github.Issue import Issue
from github.Label import Label
from github.Milestone import Milestone
from github.NamedUser import NamedUser
from github.PullRequest import PullRequest
from github.Repository import Repository
from sorcery import assigned_names


# TODO: CONVERT TO PLUGINS for heimdallr!


class RepoTypeEnum(Enum):
    all, public, private, forks, sources, member = assigned_names()
    internal = "internal"  # TODO: maybe not be supported


class RepoSortEnum(Enum):
    created, updated, pushed, full_name = assigned_names()


class DirectionEnum(Enum):
    asc, desc = assigned_names()


class StateEnum(Enum):
    open, closed, all = assigned_names()


class PullRequestSortEnum(Enum):
    (created, updated, popularity) = assigned_names()  # (comment  count),

    long_running = (
        "long-running"  # (age, filtering by pulls  updated in the  last  month).
    )


class IssueSortEnum(Enum):
    created, updated, comments = assigned_names()


def yield_org_repos(
    g: Github,
    org_name: str,
    repo_type: Union[str, RepoTypeEnum] = RepoTypeEnum.all,
    sort: Union[str, RepoSortEnum] = RepoSortEnum.pushed,
    direction: Union[str, DirectionEnum] = DirectionEnum.desc,
) -> Generator[Repository, None, None]:
    """

    Read more: https://docs.github.com/en/rest/reference/repos#list-organization-repositories--parameters

    Args:
      g:
      org_name:
      repo_type:
      sort:
      direction:

    Returns:

    """
    repo_type = RepoTypeEnum(repo_type)
    sort = RepoSortEnum(sort)
    direction = DirectionEnum(direction)
    yield from g.get_organization(org_name).get_repos(
        type=repo_type.value, sort=sort.value, direction=direction.value
    )


def yield_org_issues(g: Github, org_name: str) -> Generator[Issue, None, None]:
    """

    :param g:
    :param org_name:
    """
    yield from yield_issues(yield_org_repos(g, org_name))


def yield_org_prs(g: Github, org_name: str) -> Generator[PullRequest, None, None]:
    """

    :param g:
    :param org_name:
    """
    yield from yield_prs(yield_org_repos(g, org_name))


def yield_issues(
    iterable: Iterable[Repository],
    milestone: Union[Milestone, str] = NotSet,
    state: Union[str, StateEnum] = StateEnum.open,
    assignee: Union[NamedUser, str] = NotSet,
    mentioned: NamedUser = NotSet,
    labels: Union[List[str], List[Label]] = NotSet,
    sort: Union[str, IssueSortEnum] = IssueSortEnum.updated,
    direction: Union[str, DirectionEnum] = DirectionEnum.desc,
    since: datetime = NotSet,  # datetime.now(),
    creator: Union[str, NamedUser] = NotSet,
) -> Generator[Issue, None, None]:
    """

    Read more: https://docs.github.com/en/rest/reference/issues#list-issues-assigned-to-the-authenticated-user--parameters

    Args:
      iterable:
      milestone:
      state:
      assignee:
      mentioned:
      labels:
      sort:
      direction:
      since:
      creator:

    Returns:

    """
    state = StateEnum(state)
    sort = IssueSortEnum(sort)
    direction = DirectionEnum(direction)
    for repo in iterable:
        yield from repo.get_issues(
            milestone=milestone,
            state=state.value,
            assignee=assignee,
            mentioned=mentioned,
            labels=labels,
            sort=sort.value,
            direction=direction.value,
            since=since,
            creator=creator,
        )


def yield_prs(
    iterable: Iterable[Repository],
    state: Union[str, StateEnum] = StateEnum.open,
    sort: Union[str, PullRequestSortEnum] = PullRequestSortEnum.updated,
    direction: Union[str, DirectionEnum] = DirectionEnum.desc,
    base: str = NotSet,
    head: str = NotSet,
) -> Generator[PullRequest, None, None]:
    """

    Read more: https://docs.github.com/en/rest/reference/pulls#list-pull-requests--parameters

    Args:
      iterable:
      state:
      sort:
      direction:
      base:
      head:

    Returns:

    """
    state = StateEnum(state)
    sort = PullRequestSortEnum(sort)
    direction = DirectionEnum(direction)
    for repo in iterable:
        yield from repo.get_pulls(
            state=state.value,
            sort=sort.value,
            direction=direction.value,
            base=base,
            head=head,
        )


#
def get_pull_request(g, org_name, repo_name, pull_request_number):
    """

    :param g:
    :param org_name:
    :param repo_name:
    :param pull_request_number:
    :return:
    """
    return (
        g.get_organization(org_name).get_repo(repo_name).get_pull(pull_request_number)
    )


import calendar


def utc_to_epoch(timestamp):  # Timestamp is a datetime object in UTC time
    """

    :param timestamp:
    :return:
    """
    return calendar.timegm(timestamp.utctimetuple())


if __name__ == "__main__":
    from heimdallr.configuration.heimdallr_settings import HeimdallrSettings

    gthb = Github(HeimdallrSettings().github_token)
    remaining = gthb.rate_limiting[0]
    reset_seconds = gthb.rate_limiting_resettime - utc_to_epoch(datetime.utcnow())
    interval_seconds = ((reset_seconds + 1) / (remaining + 1)) + 1
    print(f"intervals recommended {interval_seconds} seconds")
    if remaining > 0 and False:
        # g = Github(base_url="https://{hostname}/api/v3", login_or_token="access_token") # Github Enterprise with custom hostname
        # print(list(yield_org_repos(gthb, 'aivclab')))
        print(list(yield_org_issues(gthb, "aivclab"))[0].repository.name)
    else:
        print("rate limited")

        print(f"reset in {timedelta(seconds=reset_seconds)} ")
    # print(list(yield_org_prs(gthb, 'aivclab')))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 17/03/2020
           """

import getpass
import inspect
import os
import shelve
from enum import Enum
from typing import Union

from warg.os_platform import is_nix

if is_nix():
    try:
        import sh
    except (ImportError, ModuleNotFoundError) as e:
        print(e)
        print("Please install sh and dependencies")

from sorcery import assigned_names

from apppath import ensure_existence
from heimdallr import PROJECT_APP_PATH
from warg import PropertySettings, is_windows

__all__ = [
    "HeimdallrSettings",  # Setting Class
    "SettingScopeEnum",  # Setting scope Enum
    "set_all_heimdallr_settings",  # Set settings function
]


# PropertySettings.raise_exception_on_none = False


class SettingScopeEnum(Enum):
    """ """

    user, site, root = assigned_names()


class EnvironmentVariablePreference(Enum):
    """ """

    prefer, if_missing, ignore = assigned_names()


class HeimdallrSettings(PropertySettings):
    """ """

    _setting_scope = None
    _google_settings_path = None
    _mqtt_settings_path = None
    _github_settings_path = None
    _credentials_base_path = None
    _teams_settings_path = None
    _look_up_env_on_missing = EnvironmentVariablePreference.if_missing

    # noinspection PyMissingConstructor
    def __init__(self, setting_scope: SettingScopeEnum = SettingScopeEnum.user):
        """Protects from overriding on initialisation"""
        # super().__init__()
        # TODO: FIGURE OUT A WAY TO EASILY COPY SETTINGS TO ROOT; for services
        # print(f'Using settings from {PROJECT_APP_PATH.user_config}')

        _setting_scope = setting_scope
        if setting_scope == SettingScopeEnum.user or is_windows():
            p = PROJECT_APP_PATH.user_config
        elif setting_scope == SettingScopeEnum.site and is_nix():
            p = PROJECT_APP_PATH.site_config
            if not p.exists():
                with sh.contrib.sudo(
                    password=getpass.getpass(
                        prompt=f"[sudo] password for {getpass.getuser()}: "
                    ),
                    _with=True,
                ):
                    sh.chown(
                        f"{getpass.getuser()}:", PROJECT_APP_PATH.site_config
                    )  # If a colon but no group name follows the user name, that user is made the owner of the files and the group of the files is changed to that user's login group.
        elif setting_scope == SettingScopeEnum.root and is_nix():
            p = PROJECT_APP_PATH.root_config
            if not HeimdallrSettings._credentials_base_path.exists():
                with sh.contrib.sudo(
                    password=getpass.getpass(
                        prompt=f"[sudo] password for {getpass.getuser()}: "
                    ),
                    _with=True,
                ):
                    sh.chown(
                        f"{getpass.getuser()}:", PROJECT_APP_PATH.root_config
                    )  # If a colon but no group name follows the user name, that user is made the owner of the files and the group of the files is changed to that user's login group.
        else:
            raise ValueError(
                "Invalid setting scope",
                setting_scope,
            )

        HeimdallrSettings._credentials_base_path = ensure_existence(p / "credentials")
        HeimdallrSettings._mqtt_settings_path = ensure_existence(p / "google.settings")
        HeimdallrSettings._google_settings_path = ensure_existence(p / "mqtt.settings")
        HeimdallrSettings._github_settings_path = ensure_existence(
            p / "github.settings"
        )
        HeimdallrSettings._teams_settings_path = ensure_existence(p / "teams.settings")

    @property
    def teams_config(self) -> Union[object, dict, None]:
        """ """
        key = inspect.currentframe().f_code.co_name
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.prefer
            and key in os.environ
        ):
            return os.environ.get(key)
        with shelve.open(str(HeimdallrSettings._teams_settings_path)) as d:
            if key in d:
                return d[key]
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.if_missing
            and key in os.environ
        ):
            return os.environ.get(key)
        return None

    @teams_config.setter
    def teams_config(self, config: dict) -> None:
        """

        Args:
            config ():

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._teams_settings_path), writeback=True
        ) as d:
            d[key] = config

    @property
    def google_calendar_id(self) -> Union[object, str, None]:
        """ """
        key = inspect.currentframe().f_code.co_name
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.prefer
            and key in os.environ
        ):
            return os.environ.get(key)
        with shelve.open(str(HeimdallrSettings._google_settings_path)) as d:
            if key in d:
                return d[key]
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.if_missing
            and key in os.environ
        ):
            return os.environ.get(key)
        return None

    @google_calendar_id.setter
    def google_calendar_id(self, calendar_id: str) -> None:
        """

        Args:
            calendar_id ():

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._google_settings_path), writeback=True
        ) as d:
            d[key] = calendar_id

    @google_calendar_id.deleter
    def google_calendar_id(self) -> None:
        """

        Args:
            calendar_id ():

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._github_settings_path), writeback=True
        ) as d:
            del d[key]

    @property
    def github_token(self) -> Union[object, str, None]:
        """ """
        key = inspect.currentframe().f_code.co_name
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.prefer
            and key in os.environ
        ):
            return os.environ.get(key)
        with shelve.open(str(HeimdallrSettings._github_settings_path)) as d:
            if key in d:
                return d[key]
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.if_missing
            and key in os.environ
        ):
            return os.environ.get(key)
        return None

    @github_token.setter
    def github_token(self, calendar_id: str) -> None:
        """

        Args:
            calendar_id ():

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._github_settings_path), writeback=True
        ) as d:
            d[key] = calendar_id

    @github_token.deleter
    def github_token(self) -> None:
        """

        Args:
            calendar_id ():

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._github_settings_path), writeback=True
        ) as d:
            del d[key]

    '''
@property
def mqtt_access_token(self) -> Optional[str]:
  """ """
  key = inspect.currentframe().f_code.co_name
  if self._look_up_env_on_missing == EnvironmentVariablePreference.prefer and key in os.environ:
          return os.environ.get(key)
  with shelve.open(str(HeimdallrSettings._mqtt_settings_path)) as d:
      if key in d:
          return d[key]
  if self._look_up_env_on_missing == EnvironmentVariablePreference.if_missing and key in os.environ:
      return os.environ.get(key)
  return None

@mqtt_access_token.setter
def mqtt_access_token(self, token: str) -> None:
  key = inspect.currentframe().f_code.co_name
  with shelve.open(str(HeimdallrSettings._mqtt_settings_path), writeback=True) as d:
      d[key] = token
'''

    @property
    def mqtt_username(self) -> Union[object, str, None]:
        """ """
        key = inspect.currentframe().f_code.co_name
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.prefer
            and key in os.environ
        ):
            return os.environ.get(key)
        with shelve.open(str(HeimdallrSettings._mqtt_settings_path)) as d:
            if key in d:
                return d[key]
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.if_missing
            and key in os.environ
        ):
            return os.environ.get(key)
        return None

    @mqtt_username.setter
    def mqtt_username(self, username: str) -> None:
        """

        Args:
            username ():

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._mqtt_settings_path), writeback=True
        ) as d:
            d[key] = username

    @mqtt_username.deleter
    def mqtt_username(self) -> None:
        """

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._mqtt_settings_path), writeback=True
        ) as d:
            del d[key]

    @property
    def mqtt_password(self) -> Union[object, str, None]:
        """ """
        key = inspect.currentframe().f_code.co_name
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.prefer
            and key in os.environ
        ):
            return os.environ.get(key)
        with shelve.open(str(HeimdallrSettings._mqtt_settings_path)) as d:
            if key in d:
                return d[key]
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.if_missing
            and key in os.environ
        ):
            return os.environ.get(key)
        return None

    @mqtt_password.setter
    def mqtt_password(self, password: str) -> None:
        """

        Args:
            password ():

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._mqtt_settings_path), writeback=True
        ) as d:
            d[key] = password

    @mqtt_password.deleter
    def mqtt_password(self) -> None:
        """

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._mqtt_settings_path), writeback=True
        ) as d:
            del d[key]

    @property
    def mqtt_broker(self) -> Union[object, str, None]:
        """ """
        key = inspect.currentframe().f_code.co_name
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.prefer
            and key in os.environ
        ):
            return os.environ.get(key)
        with shelve.open(str(HeimdallrSettings._mqtt_settings_path)) as d:
            if key in d:
                return d[key]
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.if_missing
            and key in os.environ
        ):
            return os.environ.get(key)
        return None

    @mqtt_broker.setter
    def mqtt_broker(self, broker: str) -> None:
        """

        Args:
            broker ():

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._mqtt_settings_path), writeback=True
        ) as d:
            d[key] = broker

    @mqtt_broker.deleter
    def mqtt_broker(self) -> None:
        """

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._mqtt_settings_path), writeback=True
        ) as d:
            del d[key]

    @property
    def mqtt_port(self) -> Union[object, str, None]:
        """ """
        key = inspect.currentframe().f_code.co_name
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.prefer
            and key in os.environ
        ):
            return os.environ.get(key)
        with shelve.open(str(HeimdallrSettings._mqtt_settings_path)) as d:
            if key in d:
                return d[key]
        if (
            self._look_up_env_on_missing == EnvironmentVariablePreference.if_missing
            and key in os.environ
        ):
            return os.environ.get(key)
        return None

    @mqtt_port.setter
    def mqtt_port(self, port: int) -> None:
        """

        Args:
            port ():

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._mqtt_settings_path), writeback=True
        ) as d:
            d[key] = port

    @mqtt_port.deleter
    def mqtt_port(self) -> None:
        """

        Returns:

        """
        key = inspect.currentframe().f_code.co_name
        with shelve.open(
            str(HeimdallrSettings._mqtt_settings_path), writeback=True
        ) as d:
            del d[key]


def set_all_heimdallr_settings(
    setting_scope: SettingScopeEnum = SettingScopeEnum.user,
    *,
    _lower_keys: bool = True,
    **kwargs,
):
    """ """
    HEIMDALLR_SETTINGS = HeimdallrSettings(setting_scope)
    # print(f"current heimdallr settings: {HEIMDALLR_SETTINGS}")
    if _lower_keys:
        kwargs = {k.lower(): v for k, v in kwargs.items()}

    # for k in kwargs.keys():
    #    assert k in HEIMDALLR_SETTINGS, f'"{k}" is not in Heimdallrs settings'

    for k in HEIMDALLR_SETTINGS:
        assert k in kwargs.keys(), f'Missing "{k}" from kwargs'

    HEIMDALLR_SETTINGS.__from_dict__(kwargs)

    print(f"new heimdallr settings: {HeimdallrSettings()}")


if __name__ == "__main__":
    settings = HeimdallrSettings()

    for k in settings:
        print(k, settings[k])

    """
set_all_heimdallr_settings(
  **{
      k: getattr(settings, k) if getattr(settings, k) else None
      for k in iter(settings)
  }
)
"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = ""

import os

from warg import NOD

SERVER_ADDRESS = "localhost"
SERVER_PORT = int(os.environ.get("PORT", 5555))

TABLE_PAGE_SIZE = 30
TIMEOUT_MACHINES_SEC = 20  # SECONDS

MQTT_TOPIC = "v1/gpu/status"
MQTT_PUBLISH_INTERVAL_SEC = 2  # SECONDS
MQTT_QOS = 0  # At most once (0), At least once (1), Exactly once (2)

HTML_TITLE = "VCLab Board"

GPU_GRAPHS_ID = "gpu-graphs"
GPU_TABLES_ID = "gpu-tables"
GPU_INTERVAL_ID = "gpu-interval"
GPU_INTERVAL_MS = MQTT_PUBLISH_INTERVAL_SEC * 1000

DU_TABLES_ID = "du-tables"
DU_INTERVAL_ID = "du-interval"
DU_INTERVAL_MS = MQTT_PUBLISH_INTERVAL_SEC * 1000 * 10

TEAMS_STATUS_ID = "teams-status"
TEAMS_STATUS_INTERVAL_ID = "teams-status-interval"
TEAMS_STATUS_INTERVAL_MS = MQTT_PUBLISH_INTERVAL_SEC * 1000 * 10

INT_COLUMNS = ["device_idx", "pid"]  # "cpu_num", "num_threads"]
PERCENT_COLUMNS = ["cpu_percent", "memory_percent"]
MB_COLUMNS = ["used_gpu_mem"]
DROP_COLUMNS = ["cmdline", "cpu_percent", "memory_percent"]

TIME_ID = "time-text"
TIME_INTERVAL_ID = "time-interval"

CALENDAR_INTERVAL_ID = "calender-interval"
CALENDAR_ID = "calender-table"
CALENDAR_INTERVAL_MS = 1000 * 60 * 60

BUILD_STATUS_MAPPING = NOD(
    streamserver="https://travis-ci.org/aivclab/streamserver.svg?branch=master",
    dlcourse="https://github.com/aivclab/dlcourse.svg?branch=master",
    poserecorder="https://github.com/aivclab/pose-recorder.svg",
    vision="https://travis-ci.com/aivclab/vision.svg?branch=master",
    RayKarsten="https://github.com/aivclab/RayKarstenWebAsm.svg",
    dmr="https://github.com/aivclab/dmr.svg",
)
BUILD_STATUS_INTERVAL = "build-status-interval"

DEBUG = True

ALL_CONSTANTS = NOD(locals())
